package com.fastbee.framework.interceptor;


import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.fastbee.common.holder.ProjectHolder;
import com.fastbee.common.holder.ProjectInfo;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.annotation.Nullable;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * @author mijiupro
 */

@Component
@Slf4j
public class ProjectDataIsolationInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(@NotNull HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull Object handler) throws Exception {

        log.info("进入项目数据隔离拦截器");
        //放行预检请求
        if ("OPTIONS".equalsIgnoreCase(Objects.requireNonNull(request.getMethod()))) {
            return true;
        }
        if ("DELETE".equalsIgnoreCase(request.getMethod())) {
            return true;
        }
        if ("POST".equalsIgnoreCase(request.getMethod())) {
                return true;
            }
        if ("PUT".equalsIgnoreCase(request.getMethod())) {
            return true;
        }
        //当前项目信息
        ProjectInfo projectInfo = ProjectInfo.builder().build();
        //获取cookie
        Cookie[] requestCookies = request.getCookies();
        //获取cookie中的projectIds（父项目及其子项目id）,projectId(当前项目id),userId（当前管理员id）
        if (requestCookies != null) {
            for (Cookie cookie : requestCookies){
                //获取cookie中的projectIds
                if ("projectIds".equals(cookie.getName())) {
                    byte[] decode = Base64.decode(cookie.getValue());
                    //转成List集合
                    String decodedStr = new String(decode);
                    if (StringUtils.isNotBlank(decodedStr)) {
                        JSONArray objects = JSONUtil.parseArray(decodedStr);
                        projectInfo.setProjectIdList(objects.toList(Long.class));
                        }
                    }
                //获取cookie中的projectId
                if ("projectId".equals(cookie.getName())) {
                    projectInfo.setProjectId(cookie.getValue());
                }
                //获取cookie中的userId
                if ("userId".equals(cookie.getName())) {
                    projectInfo.setProjectAdminId(cookie.getValue());
                }

            }

        }
        ProjectHolder.setProjectInfo(projectInfo);

//        //获取路径中的查询字符串参数
//        String projectIds = request.getParameter("projectIds");
//
//        //为空则是超级管理员不做数据隔离
//        if (StringUtils.isBlank(projectIds)) {
//                ProjectHolder.setProjectInfo(ProjectInfo.builder().projectId("").build());
//                return true;
//        }
//            byte[] decode = Base64.decode(projectIds);
//            //转成List集合
//            String decodedStr = new String(decode);
//            JSONArray objects = JSONUtil.parseArray(decodedStr);
//            ProjectHolder.setProjectInfo(ProjectInfo.builder()
//                    .projectIdList(objects.toList(Long.class))
//                    .build());
            return true;
    }
    @Override
    public void afterCompletion(@NotNull HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull Object handler, @Nullable Exception ex)  {
        // 清理本地线程
        ProjectHolder.clear();
    }
}