package com.fastbee.system.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.system.mapper.SysDistrictMapper;
import com.fastbee.system.domain.SysDistrict;
import com.fastbee.system.service.ISysDistrictService;


/**
 * 行政区划Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-10-18
 */
@Service
public class SysDistrictServiceImpl implements ISysDistrictService 
{
    @Autowired
    private SysDistrictMapper sysDistrictMapper;

    /**
     * 查询行政区划
     * 
     * @param id 行政区划主键
     * @return 行政区划
     */
    @Override
    public SysDistrict selectSysDistrictById(Long id)
    {
        return sysDistrictMapper.selectSysDistrictById(id);
    }

    /**
     * 查询行政区划列表
     * 
     * @param sysDistrict 行政区划
     * @return 行政区划
     */
    @Override
    public List<SysDistrict> selectSysDistrictList(SysDistrict sysDistrict)
    {
        return sysDistrictMapper.selectSysDistrictList(sysDistrict);
    }

    /**
     * 新增行政区划
     * 
     * @param sysDistrict 行政区划
     * @return 结果
     */
    @Override
    public int insertSysDistrict(SysDistrict sysDistrict)
    {
        return sysDistrictMapper.insertSysDistrict(sysDistrict);
    }

    /**
     * 修改行政区划
     * 
     * @param sysDistrict 行政区划
     * @return 结果
     */
    @Override
    public int updateSysDistrict(SysDistrict sysDistrict)
    {
        return sysDistrictMapper.updateSysDistrict(sysDistrict);
    }

    /**
     * 批量删除行政区划
     * 
     * @param ids 需要删除的行政区划主键
     * @return 结果
     */
    @Override
    public int deleteSysDistrictByIds(Long[] ids)
    {
        return sysDistrictMapper.deleteSysDistrictByIds(ids);
    }

    /**
     * 删除行政区划信息
     * 
     * @param id 行政区划主键
     * @return 结果
     */
    @Override
    public int deleteSysDistrictById(Long id)
    {
        return sysDistrictMapper.deleteSysDistrictById(id);
    }

    /**
     * 查询行政区划树状列表
     * @return 树状列表
     */
    @Override
    public List<SysDistrict> selectSysDistrictTree() {
        List<SysDistrict> sysDistrictList = new LambdaQueryChainWrapper<>(sysDistrictMapper)
                .select(SysDistrict::getId, SysDistrict::getParentId, SysDistrict::getName, SysDistrict::getAdcode,
                        SysDistrict::getLevel, SysDistrict::getFullName, SysDistrict::getName
                        , SysDistrict::getLng, SysDistrict::getLat)
                .list();
        return buildTree(sysDistrictList);

    }

    private List<SysDistrict> buildTree(List<SysDistrict> SysDistricts) {
        Map<Long, SysDistrict> map = new HashMap<>();
        List<SysDistrict> roots = new ArrayList<>();
        for (SysDistrict sysDistrict : SysDistricts) {
            map.put(sysDistrict.getAdcode(), sysDistrict);
        }
        return getDistricts(SysDistricts, map, roots);
    }

    private List<SysDistrict> getDistricts(List<SysDistrict> SysDistricts, Map<Long, SysDistrict> map, List<SysDistrict> roots) {
        for (SysDistrict sysDistrict : SysDistricts) {
            Long parentId = sysDistrict.getParentId();
            if (parentId == null || !map.containsKey(parentId)) {
                roots.add(sysDistrict);
            } else {
                SysDistrict parent = map.get(parentId);
                if (parent.getChildren() == null) {
                    parent.setChildren(new ArrayList<>());
                }
                parent.getChildren().add(sysDistrict);
            }
        }
        return roots;
    }

}
